﻿using System;
using System.Linq;
using System.Runtime.InteropServices;
using Actian.Data.Zen;

namespace btest
{
    class Program
    {
        static string btrieveFileName = "squaresAndSquareRoots.btr";

        const int MIN_X = 0;
        const int MAX_X = 255;

        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        struct record_t
        {
            public byte x;
            public UInt16 xSquared;
            public double xSquareRoot;
        };

        public static byte[] Serialize<T>(T s) where T : struct
        {
            var size = Marshal.SizeOf(typeof(T));
            var array = new byte[size];
            var ptr = Marshal.AllocHGlobal(size);
            Marshal.StructureToPtr(s, ptr, true);
            Marshal.Copy(ptr, array, 0, size);
            Marshal.FreeHGlobal(ptr);
            return array;
        }

        public static T Deserialize<T>(byte[] array) where T : struct
        {
            var size = Marshal.SizeOf(typeof(T));
            var ptr = Marshal.AllocHGlobal(size);
            Marshal.Copy(array, 0, ptr, size);
            var s = (T)Marshal.PtrToStructure(ptr, typeof(T));
            Marshal.FreeHGlobal(ptr);
            return s;
        }

        static Btrieve.StatusCode createFile(ref BtrieveClient btrieveClient)
        {
            Btrieve.StatusCode status;
            BtrieveFileAttributes btrieveFileAttributes = new BtrieveFileAttributes();
            record_t record = new record_t();

            // If SetFixedRecordLength() fails.
            if ((status = btrieveFileAttributes.SetFixedRecordLength(Marshal.SizeOf(record))) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveFileAttributes.SetFixedRecordLength():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            // If FileCreate() fails.
            if ((status = btrieveClient.FileCreate(btrieveFileAttributes, btrieveFileName, Btrieve.CreateMode.CREATE_MODE_OVERWRITE)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveClient.FileCreate():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode openFile(ref BtrieveClient btrieveClient, ref BtrieveFile btrieveFile)
        {
            Btrieve.StatusCode status;

            // If FileOpen() fails.
            if ((status = btrieveClient.FileOpen(btrieveFile, btrieveFileName, null, Btrieve.OpenMode.OPEN_MODE_NORMAL)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveClient.FileOpen():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode loadFile(ref BtrieveFile btrieveFile)
        {
            Btrieve.StatusCode status = Btrieve.StatusCode.STATUS_CODE_NO_ERROR;
            int i;
            record_t record = new record_t();
            byte[] recordBytes;

            // For all the values of x.
            for (i = MIN_X; i <= MAX_X; i++)
            {
                record.x = (byte)i;
                record.xSquared = (UInt16)(i * i);
                record.xSquareRoot = Math.Sqrt((double)i);
                recordBytes = Serialize(record);

                // If RecordCreate() fails.
                if ((status = btrieveFile.RecordCreate(recordBytes, Marshal.SizeOf(record))) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
                {
                    Console.WriteLine("Error: BtrieveFile.RecordCreate():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                    goto leave;
                }
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode closeFile(ref BtrieveClient btrieveClient, ref BtrieveFile btrieveFile)
        {
            Btrieve.StatusCode status;

            // If FileClose() fails.
            if ((status = btrieveClient.FileClose(btrieveFile)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveClient.FileClose():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode deleteFile(ref BtrieveClient btrieveClient)
        {
            Btrieve.StatusCode status;

            // If FileDelete() fails.
            if ((status = btrieveClient.FileDelete(btrieveFileName)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveClient.FileDelete():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode createIndex(ref BtrieveFile btrieveFile)
        {
            Btrieve.StatusCode status;
            BtrieveIndexAttributes btrieveIndexAttributes = new BtrieveIndexAttributes();
            BtrieveKeySegment btrieveKeySegment = new BtrieveKeySegment();

            // If SetField() fails.
            if ((status = btrieveKeySegment.SetField(0, 1, Btrieve.DataType.DATA_TYPE_UNSIGNED_BINARY)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveKeySegment.SetField():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            // If AddKeySegment() fails.
            if ((status = btrieveIndexAttributes.AddKeySegment(btrieveKeySegment)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveIndexAttributes.AddKeySegment():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            // If IndexCreate() fails.
            if ((status = btrieveFile.IndexCreate(btrieveIndexAttributes)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                Console.WriteLine("Error: BtrieveFile.IndexCreate():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            leave:
            return status;
        }

        static Btrieve.StatusCode retrieveRecord(ref BtrieveFile btrieveFile, ref byte key)
        {
            Btrieve.StatusCode status = Btrieve.StatusCode.STATUS_CODE_NO_ERROR;
            record_t record = new record_t();
            byte[] recordBytes = Serialize(record);
            byte[] keyBytes = Serialize(key);

            // If RecordRetrieve() fails.
            if (btrieveFile.RecordRetrieve(Btrieve.Comparison.COMPARISON_EQUAL, Btrieve.Index.INDEX_1, keyBytes, Marshal.SizeOf(key), recordBytes, Marshal.SizeOf(record)) != Marshal.SizeOf(record))
            {
                status = btrieveFile.GetLastStatusCode();
                Console.WriteLine("Error: BtrieveFile.RecordRetrieve():{0}:{1}.\n", status, Btrieve.StatusCodeToString(status));
                goto leave;
            }

            record = Deserialize<record_t>(recordBytes);
            Console.WriteLine("record: ({0}, {1}, {2})\n", record.x, record.xSquared, record.xSquareRoot);

            leave:
            return status;
        }

        static int Main(string[] args)
        {
            BtrieveClient btrieveClient = new BtrieveClient();
            Btrieve.StatusCode status = Btrieve.StatusCode.STATUS_CODE_UNKNOWN;
            BtrieveFile btrieveFile = new BtrieveFile();
            byte key;
            UInt64 integerValue;

            // If the incorrect number of arguments were given.
            if (args.Count() != 1)
            {
                Console.WriteLine("Usage: {0} uint8_value\n", Environment.GetCommandLineArgs()[0]);
                goto leave;
            }

            integerValue = UInt64.Parse(args[0]);

            // If integerValue is out of range.
            if ((integerValue < MIN_X) || (integerValue > MAX_X))
            {
                Console.WriteLine("Usage: {0} uint8_value\n", Environment.GetCommandLineArgs()[0]);
                goto leave;
            }

            key = (byte)integerValue;

            // If createFile() fails.
            if ((status = createFile(ref btrieveClient)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If openFile() fails.
            if ((status = openFile(ref btrieveClient, ref btrieveFile)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If loadFile() fails.
            if ((status = loadFile(ref btrieveFile)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If createIndex() fails.
            if ((status = createIndex(ref btrieveFile)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If retrieveRecord() fails.
            if ((status = retrieveRecord(ref btrieveFile, ref key)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If closeFile() fails.
            if ((status = closeFile(ref btrieveClient, ref btrieveFile)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            // If deleteFile() fails.
            if ((status = deleteFile(ref btrieveClient)) != Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
            {
                goto leave;
            }

            leave:
            // If there wasn't a failure.
            if (status == Btrieve.StatusCode.STATUS_CODE_NO_ERROR)
                return 0;

            return 1;
        }
    }
}
