import sys
import struct
import math
import os
import platform

if (platform.system() == "Windows"):
    sys.path.append("C:\\Program Files\\Actian\\Zen\\bin")
else:
    sys.path.append("/usr/local/actianzen/lib64")

import btrievePython

min_x = 0
max_x = 255
btrieveFileName = "squaresAndSquareRoots.btr"
recordFormat = "<BHd"
recordLength = 11
keyFormat = "<B"

if (len(sys.argv) != 2):
    sys.exit("Usage: " + os.path.basename(sys.argv[0]) + " uint8_value")

integerValue = int(sys.argv[1])

if (integerValue < min_x) or (integerValue > max_x):
    sys.exit("Usage: " + os.path.basename(sys.argv[0]) + " uint8_value")

btrieveClient = btrievePython.BtrieveClient()
assert(btrieveClient != None)

btrieveFileAttributes = btrievePython.BtrieveFileAttributes()
assert(btrieveFileAttributes != None)

rc = btrieveFileAttributes.SetFixedRecordLength(recordLength)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveFileAttributes::SetLogicalFixedRecordLength():%d,%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

rc = btrieveClient.FileCreate(btrieveFileAttributes, btrieveFileName, btrievePython.Btrieve.CREATE_MODE_OVERWRITE)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveClient::FileCreate():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

btrieveFile = btrievePython.BtrieveFile()
assert(btrieveFile != None)

rc = btrieveClient.FileOpen(btrieveFile, btrieveFileName, None, btrievePython.Btrieve.OPEN_MODE_NORMAL)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveClient::FileOpen():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

# For all the values of x.
for i in range(min_x, max_x + 1):
    record = struct.pack(recordFormat, i, i * i, math.sqrt(i))

    rc = btrieveFile.RecordCreate(record)
    assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveFile::RecordCreate():%d:%s.\n"  % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

btrieveKeySegment = btrievePython.BtrieveKeySegment()
assert(btrieveKeySegment != None)

rc = btrieveKeySegment.SetField(0, 1, btrievePython.Btrieve.DATA_TYPE_UNSIGNED_BINARY)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveKeySegment::SetField():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

btrieveIndexAttributes = btrievePython.BtrieveIndexAttributes()
assert(btrieveIndexAttributes != None), "Error: BtrieveIndexAttributes::AddKeySegment():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

rc = btrieveIndexAttributes.AddKeySegment(btrieveKeySegment)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveIndexAttributes::AddKeySegment():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

rc = btrieveFile.IndexCreate(btrieveIndexAttributes)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveFile::IndexCreate():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

# print record
record = struct.pack(recordFormat, 0, 0, 0)
key = struct.pack(keyFormat, integerValue)

rc = btrieveFile.RecordRetrieve(btrievePython.Btrieve.COMPARISON_EQUAL,btrievePython.Btrieve.INDEX_1, key, record)
assert(rc == recordLength), "Error: BtrieveFileRecordRetrieveEqual():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(btrieveFile.GetLastStatusCode()))

print ("record:", struct.unpack(recordFormat, record))

rc = btrieveClient.FileClose(btrieveFile)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveClient::FileClose():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))

rc = btrieveClient.FileDelete(btrieveFileName)
assert(rc == btrievePython.Btrieve.STATUS_CODE_NO_ERROR), "Error: BtrieveClient::FileDelete():%d:%s.\n" % (rc, btrievePython.Btrieve_StatusCodeToString(rc))
